/*
 * EmailMessage.java
 *
 * Created on 24 March 2005, 15:17
 */

package jim.common.mail;

import java.util.HashMap;
import java.util.Map;

import javax.mail.Address;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.Message.RecipientType;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import jim.common.Logger;
import jim.common.StringFormatter;
import jim.common.Trace;

import org.apache.commons.logging.Log;

/**
 * An email message (template) that could be sent multiple times to multiple recipients.
 *
 * The message can be tailored for each recipient.
 * @author briggsj
 */
public class EmailMessage {
    
    static private Log log = Logger.getLogger(EmailMessage.class);
    
    /** Creates a new instance of EmailMessage */
    public EmailMessage() {
    }
    
    /**
     * Holds value of property sender.
     */
    private String sender;
    
    /**
     * Holds value of property text.
     */
    private String text;
    
    /**
     * Holds value of property subject.
     */
    private String subject;
    
    /**
     * Holds value of property generalParams. Args that are common to all messages sent from this template
     */
    private Map<String,String> generalParams;
    
    /**
     * Holds value of property session.
     */
    private Session session;
    
    /**
     * Holds value of property tailoredParams. Args that are specific to a single message send
     */
    private Map<String,String> tailoredParams;
    
    /**
     * Getter for property sender.
     * @return Value of property sender.
     */
    public String getSender() {
        return this.sender;
    }
    
    /**
     * Setter for property sender.
     * @param sender New value of property sender.
     */
    public void setSender(String sender) {
        this.sender = sender;
    }
    
    /**
     * Getter for property text.
     * @return Value of property text.
     */
    public String getText() {
        return this.text;
    }
    
    /**
     * Setter for property text.
     * @param text New value of property text.
     */
    public void setText(String text) {
        this.text = text;
    }
    
    /**
     * Getter for property subject.
     * @return Value of property subject.
     */
    public String getSubject() {
        return this.subject;
    }
    
    /**
     * Setter for property subject.
     * @param subject New value of property subject.
     */
    public void setSubject(String subject) {
        this.subject = subject;
    }
    
    /**
     * Getter for property generalParams.
     * @return Value of property generalParams.
     */
    public Map getGeneralParams() {
        return this.generalParams;
    }
    
    /**
     * Setter for property generalParams.
     * @param generalParams New value of property generalParams.
     */
    public void setGeneralParams(Map<String, String> generalParams) {
        this.generalParams = generalParams;
    }
    
    /**
     * Display an email message as a string
     * @return <CODE>String</CODE>
     */
    public String toString() {
        String retValue;
        
        retValue = "\n" + "From: " + ((sender!=null)?sender:"no sender") + "\n"
                + "To: " + ((to!=null)?to:"no recipients") + "\n"
                + "Subject: " + subject + "\n"
                + "\n"
                + getMessage();
        return retValue;
    }
    
    /**
     * Get the message text. Any tailoring will be applied.
     * @return <CODE>String</CODE>
     */
    public String getMessage() {
        Map<String,String> localMap = new HashMap<String,String>(generalParams);
        if (tailoredParams == null) {
            // do nothing
        } else {
            localMap.putAll(tailoredParams);
        }
        String message = StringFormatter.apply(text, localMap);
        return message;
    }
    
    /**
     * Send email to specified people
     * @param to Comma delimited list of recipients
     * @param testing If true, the message is logged rather than sent
     * @throws MessagingException Passes on any exception thrown by the underlying transport
     * @throws AddressException Passes on any exception thrown by underlying transport
     */
    public void sendMail(String to, boolean testing) throws MessagingException, AddressException {
        MimeMessage message = new MimeMessage(this.session);
        message.setFrom(new InternetAddress(this.getSender()));
        this.setTo(to);
        Address recips [] = InternetAddress.parse(to);
        message.addRecipients(RecipientType.TO, recips);
        message.setSubject(this.getSubject());
        message.setText(this.getMessage());
        Trace.print(this.toString());
        if (testing) {
            Log log = Logger.getLogger();
            log.info(this.toString());
        } else {
            Transport.send(message);
        }
    }
    
    /**
     * Getter for property session.
     * @return Value of property session.
     */
    public Session getSession() {
        return this.session;
    }
    
    /**
     * Setter for property session.
     * @param session New value of property session.
     */
    public void setSession(Session session) {
        this.session = session;
    }
    
    /**
     * Getter for property tailoredParams.
     * @return Value of property tailoredParams.
     */
    public Map getTailoredParams() {
        return this.tailoredParams;
    }
    
    /**
     * Setter for property tailoredParams.
     * @param tailoredParams New value of property tailoredParams.
     */
    public void setTailoredParams(Map<String,String> tailoredParams) {
        this.tailoredParams = tailoredParams;
    }
    
    /**
     * Holds value of property to.
     */
    private String to;
    
    /**
     * Getter for property to.
     * @return Value of property to.
     */
    public String getTo() {
        return this.to;
    }
    
    /**
     * Setter for property to.
     * @param to New value of property to.
     */
    private void setTo(String to) {
        this.to = to;
    }
}